#!/bin/sh

#
# Copyright (c) 2013 Peter Holm <pho@FreeBSD.org>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# $FreeBSD$
#

[ `id -u ` -ne 0 ] && echo "Must be root!" && exit 1

# kern/178238 "nullfs don't release i-nodes on unlink"
# See also nullfs16.sh
# Fixed by: r292961.

. ../default.cfg

nullfsmp=${mntpoint}2
mount | grep -q "$nullfsmp " && umount $nullfsmp

[ -d $nullfsmp ] || mkdir $nullfsmp
mount | grep "$mntpoint " | grep -q /dev/md && umount -f $mntpoint
mdconfig -l | grep -q md$mdstart &&  mdconfig -d -u $mdstart
mdconfig -a -t swap -s 1g -u $mdstart
bsdlabel -w md$mdstart auto
newfs $newfs_flags md${mdstart}$part > /dev/null
mount /dev/md${mdstart}$part $mntpoint
chmod 777 $mntpoint
mount -t nullfs $mntpoint $nullfsmp

(cd $nullfsmp; jot 20 | xargs touch)
rm $nullfsmp/*

(cd $nullfsmp; jot 20 | xargs touch)
rm $mntpoint/*

(cd $mntpoint; jot 20 | xargs touch)
rm $mntpoint/*

(cd $mntpoint; jot 20 | xargs touch)
rm $nullfsmp/*

ino=`df -i $nullfsmp | tail -1 | awk '{print $6}'`
if [ $ino -ne 2 ]; then
	echo FAIL
	df -i $mntpoint $nullfsmp
	echo "ls -la $mntpoint $nullfsmp"
	ls -la $mntpoint $nullfsmp
fi

while mount | grep -q "$nullfsmp "; do
	umount $nullfsmp || sleep 1
done

while mount | grep -q "$mntpoint "; do
	umount $mntpoint || sleep 1
done
mdconfig -d -u $mdstart
